CREATE OR REPLACE PACKAGE LIGHTY_PKG
AS
  PROCEDURE snap;
  PROCEDURE purge;
  PROCEDURE set_retention(
      days NUMBER);
END LIGHTY_PKG;
/
show error

CREATE OR REPLACE PACKAGE BODY LIGHTY_PKG
AS
PROCEDURE set_retention(
    DAYS NUMBER)
IS
BEGIN
  MERGE INTO PARAMETERS USING
  (SELECT 1 FROM dual
  ) ON (PARAMETERS.NAME = 'retention')
WHEN MATCHED THEN
  UPDATE SET value=DAYS WHEN NOT MATCHED THEN
  INSERT
    (name,value
    ) VALUES
    ('retention',DAYS
    );
  COMMIT;
EXCEPTION
WHEN OTHERS THEN
  RAISE_APPLICATION_ERROR(-20100,'L-ASH Error' || SUBSTR(SQLERRM, 1 , 1000));
END set_retention;
PROCEDURE snap
IS
BEGIN
  dbms_application_info.set_module('ASHSTAT','INSERT L-ASH');
  INSERT
  INTO L_ASH
    (
      INST_ID,
      SAMPLE_TIME,
      SESSION_ID,
      SESSION_SERIAL#,
      USER_ID,
      SQL_ID,
      SQL_CHILD_NUMBER,
      SQL_OPCODE,
      SQL_PLAN_HASH_VALUE,
      BLOCKING_SESSION,
      BLOCKING_SESSION_STATUS,
      EVENT#,
      P1,
      P2,
      P3,
      WAIT_TIME,
      CURRENT_OBJ#,
      CURRENT_FILE#,
      CURRENT_BLOCK#,
      PROGRAM,
      MODULE,
      ACTION,
      CLIENT_ID,
      SERVICE_HASH,
      SESSION_TYPE,
      SESSION_STATE,
      WAIT_CLASS_ID,
      BLOCKING_INST_ID,
      SQL_EXEC_ID,
      SQL_EXEC_START,
      PLSQL_ENTRY_OBJECT_ID,
      PLSQL_ENTRY_SUBPROGRAM_ID,
      PLSQL_OBJECT_ID,
      PLSQL_SUBPROGRAM_ID,
      MACHINE
    )
  SELECT s.INST_ID,
    SYSTIMESTAMP,
    s.SID,
    s.SERIAL#,
    s.USER#,
    s.SQL_ID,
    s.SQL_CHILD_NUMBER,
    q.COMMAND_TYPE,
    q.PLAN_HASH_VALUE,
    s.BLOCKING_SESSION,
    s.BLOCKING_SESSION_STATUS,
    s.EVENT#,
    s.P1,
    s.P2,
    s.P3,
    s.WAIT_TIME,
    (
    CASE
      WHEN s.state='WAITING'
      THEN s.ROW_WAIT_OBJ#
      ELSE -1
    END),
    (
    CASE
      WHEN s.state='WAITING'
      AND s.P1text='file#'
      THEN s.P1
      ELSE -1
    END),
    (
    CASE
      WHEN s.state='WAITING'
      AND s.P2text='block#'
      THEN s.P2
      ELSE -1
    END),
    s.PROGRAM,
    s.MODULE,
    s.ACTION,
    s.CLIENT_IDENTIFIER,
    decode(d.NAME_HASH,null,0,d.NAME_HASH),
    DECODE(TYPE,'USER','FOREGROUND','BACKGROUND'),
    DECODE(state,'WAITING','WAITING','ON CPU') ,
    DECODE(state,'WAITING',s.WAIT_CLASS_ID,null) ,
    s.BLOCKING_INSTANCE,
    s.SQL_EXEC_ID,
    s.SQL_EXEC_START,
    s.PLSQL_ENTRY_OBJECT_ID,
    s.PLSQL_ENTRY_SUBPROGRAM_ID,
    s.PLSQL_OBJECT_ID,
    s.PLSQL_SUBPROGRAM_ID,
    s.MACHINE
  FROM gv$session s, dba_services d,gv$sql q
  WHERE s.status    ='ACTIVE'
  AND ((s.state='WAITING' and s.wait_class !='Idle') or (s.state != 'WAITING'))
  and audsid <> userenv('SESSIONID')
  and paddr not in (select paddr from v$bgprocess where name='CJQ0')
  and s.sql_id=q.sql_id(+)
  and s.sql_child_number=q.child_number(+)
  and s.inst_id=q.inst_id(+)
  and s.service_name=d.name(+);

  COMMIT;
EXCEPTION
WHEN OTHERS THEN
  RAISE_APPLICATION_ERROR(-20100,'L-ASH Error' || SUBSTR(SQLERRM, 1 , 1000));
END snap;
PROCEDURE purge
IS
  NB_ROWS NUMBER;
  DAYS    NUMBER := 7;
BEGIN
  dbms_application_info.set_module('ASHSTAT','PURGE L-ASH');
  SELECT value INTO days FROM parameters WHERE name='retention';
  DELETE FROM L_ASH WHERE sample_time < TRUNC(sysdate)-days;
  COMMIT;
EXCEPTION
WHEN OTHERS THEN
  RAISE_APPLICATION_ERROR(-20100,'L-ASH PURGE Error' || SUBSTR(SQLERRM, 1 , 1000));
END purge;
END LIGHTY_PKG;
/
sho err
